@@ -1,3 +1,6 @@
+# Copyright (C) 2021, QuantStack
+# SPDX-License-Identifier: BSD-3-Clause
+
 from boa.core.solver import get_solver
 import copy
 import re
@@ -19,7 +22,6 @@ from conda.base.context import context
 from conda_build.config import get_or_merge_config
 from conda_build.utils import apply_pin_expressions
 from conda.models.channel import Channel as CondaChannel
-from conda.core.package_cache_data import PackageCacheData
 from conda_build.metadata import eval_selector, ns_cfg
 
 from boa.core.config import boa_config
@@ -233,6 +235,7 @@ class Output:
             self.requirements[section] = [
                 CondaBuildSpec(r) for r in (self.requirements.get(section) or [])
             ]
+        self.sections["requirements"] = self.requirements
 
         # handle strong and weak run exports
         run_exports = []
@@ -327,6 +330,7 @@ class Output:
             "name": self.name,
             "version": self.version,
             "build_number": self.build_number,
+            "source": self.data["source"],
         }
 
         res["differentiating_variant"] = self.differentiating_variant
@@ -445,6 +449,10 @@ class Output:
             add_header("Run", True)
             for r in self.requirements["run"]:
                 spec_format(r)
+        if self.requirements["run_constrained"]:
+            add_header("Run Constraints", True)
+            for r in self.requirements["run_constrained"]:
+                spec_format(r)
         return table
 
     def __repr__(self):
@@ -574,7 +582,6 @@ class Output:
             spec_map = {s.final_name: s for s in specs}
             specs = [str(x) for x in specs]
 
-            pkg_cache = PackageCacheData.first_writable().pkgs_dir
             if env in ("host", "run") and not self.config.subdirs_same:
                 subdir = self.config.host_subdir
             else:
@@ -605,7 +612,7 @@ class Output:
             }
 
             downloaded = t.fetch_extract_packages(
-                pkg_cache, solver.repos + list(solver.local_repos.values()),
+                solver.repos + list(solver.local_repos.values()),
             )
             if not downloaded:
                 raise RuntimeError("Did not succeed in downloading packages.")
