@@ -1,6 +1,10 @@
+# Copyright (C) 2021, QuantStack
+# SPDX-License-Identifier: BSD-3-Clause
+
 import os
 import glob
 import itertools
+import json
 
 from mamba.mamba_api import PrefixData
 
@@ -11,20 +15,21 @@ from boa.core.solver import refresh_solvers
 from boa.core.build import build, download_source
 from boa.core.metadata import MetaData
 from boa.core.test import run_test
+from boa.core.config import boa_config
 
 from conda_build.utils import rm_rf
 import conda_build.jinja_context
 from conda.common import toposort
+from conda.base.context import context
 from conda.models.match_spec import MatchSpec
 from conda.gateways.disk.create import mkdir_p
 from conda_build.variants import get_default_variant
 from conda_build.utils import ensure_list
 from conda_build.index import update_index
 
-from rich.console import Console
 from rich.table import Table
 
-console = Console()
+console = boa_config.console
 
 
 def find_all_recipes(target, config):
@@ -80,12 +85,12 @@ def find_all_recipes(target, config):
         recursive_add(target)
 
     sorted_recipes = toposort.toposort(sort_recipes)
-    print(sorted_recipes)
+    console.print(sorted_recipes)
 
     return [recipes[x] for x in sorted_recipes]
 
 
-def get_dependency_variants(requirements, conda_build_config, config, features=()):
+def get_dependency_variants(requirements, conda_build_config, config):
     host = requirements.get("host") or []
     build = requirements.get("build") or []
     # run = requirements.get("run") or []
@@ -241,21 +246,85 @@ def to_build_tree(ydoc, variants, config, cbc, selected_features):
 
     final_outputs = []
 
+    # need to strip static away from output name... :/
+    static_feature = selected_features.get("static", False)
+
     for name in tsorted:
         output = outputs[name]
-        if variants.get(output.name):
-            v = variants[output.name]
-            combos = []
 
+        # this is all a bit hacky ... will have to clean that up eventually
+        variant_name = name
+        if static_feature and name.endswith("-static"):
+            variant_name = name[: -len("-static")]
+
+        # zip keys need to be contracted
+        zipped_keys = cbc.get("zip_keys", [])
+
+        if variants.get(variant_name):
+            v = variants[variant_name]
+            import copy
+
+            vzipped = copy.copy(v)
+            zippers = {}
+            for zkeys in zipped_keys:
+                # we check if our variant contains keys that need to be zipped
+                if sum(k in v for k in zkeys) > 1:
+                    filtered_zip_keys = [k for k in v if k in zkeys]
+                    print("Filtered zip keys: ", filtered_zip_keys)
+
+                    zkname = "__zip_" + "_".join(filtered_zip_keys)
+
+                    zklen = None
+                    for zk in filtered_zip_keys:
+                        if zk not in cbc:
+                            raise RuntimeError(
+                                f"Trying to zip keys, but not all zip keys found on conda-build-config {zk}"
+                            )
+
+                        zkl = len(cbc[zk])
+                        if not zklen:
+                            zklen = zkl
+
+                        if zklen and zkl != zklen:
+                            raise RuntimeError(
+                                f"Trying to zip keys, but not all zip keys have the same length {zkeys}"
+                            )
+
+                    vzipped[zkname] = [str(i) for i in range(zklen)]
+                    zippers[zkname] = {zk: cbc[zk] for zk in filtered_zip_keys}
+
+                    for zk in filtered_zip_keys:
+                        del vzipped[zk]
+
+            combos = []
             differentiating_keys = []
-            for k in v:
-                if len(v[k]) > 1:
+            for k, vz in vzipped.items():
+                if len(vz) > 1:
                     differentiating_keys.append(k)
-                combos.append([(k, x) for x in v[k]])
+                combos.append([(k, x) for x in vz])
 
             all_combinations = tuple(itertools.product(*combos))
             all_combinations = [dict(x) for x in all_combinations]
+
+            # unzip the zipped keys
+            unzipped_combinations = []
             for c in all_combinations:
+                unz_combo = {}
+                for vc in c:
+                    if vc.startswith("__zip_"):
+                        ziptask = zippers[vc]
+                        zipindex = int(c[vc])
+                        for zippkg in ziptask:
+                            unz_combo[zippkg] = ziptask[zippkg][zipindex]
+                        if vc in differentiating_keys:
+                            differentiating_keys.remove(vc)
+                            differentiating_keys.extend(zippers[vc].keys())
+                    else:
+                        unz_combo[vc] = c[vc]
+
+                unzipped_combinations.append(unz_combo)
+
+            for c in unzipped_combinations:
                 x = output.apply_variant(c, differentiating_keys)
                 final_outputs.append(x)
         else:
@@ -303,20 +372,17 @@ def to_build_tree(ydoc, variants, config, cbc, selected_features):
     return final_outputs
 
 
-def build_recipe(args, recipe_path, cbc, config, notest: bool = False):
-
-    if args.features:
-        assert args.features.startswith("[") and args.features.endswith("]")
-        features = [f.strip() for f in args.features[1:-1].split(",")]
-    else:
-        features = []
-
-    selected_features = {}
-    for f in features:
-        if f.startswith("~"):
-            selected_features[f[1:]] = False
-        else:
-            selected_features[f] = True
+def build_recipe(
+    command,
+    recipe_path,
+    cbc,
+    config,
+    selected_features,
+    notest: bool = False,
+    skip_existing: bool = False,
+    interactive: bool = False,
+    skip_fast: bool = False,
+):
 
     ydoc = render(recipe_path, config=config)
     # We need to assemble the variants for each output
@@ -338,12 +404,12 @@ def build_recipe(args, recipe_path, cbc, config, notest: bool = False):
             o["selected_features"] = selected_features
 
             variants[o["package"]["name"]] = get_dependency_variants(
-                o.get("requirements", {}), cbc, config, features
+                o.get("requirements", {}), cbc, config
             )
     else:
         # we only have one output
         variants[ydoc["package"]["name"]] = get_dependency_variants(
-            ydoc.get("requirements", {}), cbc, config, features
+            ydoc.get("requirements", {}), cbc, config
         )
 
     # this takes in all variants and outputs, builds a dependency tree and returns
@@ -359,10 +425,14 @@ def build_recipe(args, recipe_path, cbc, config, notest: bool = False):
     #   - solv build, add weak run exports to
     # - add run exports from deps!
 
-    if args.command == "render":
-        for o in sorted_outputs:
-            console.print(o)
-        return
+    if command == "render":
+        if boa_config.json:
+            jlist = [o.to_json() for o in sorted_outputs]
+            print(json.dumps(jlist, indent=4))
+        else:
+            for o in sorted_outputs:
+                console.print(o)
+        return sorted_outputs
 
     # TODO this should be done cleaner
     top_name = ydoc["package"]["name"]
@@ -372,13 +442,43 @@ def build_recipe(args, recipe_path, cbc, config, notest: bool = False):
 
     console.print("\n[yellow]Initializing mamba solver[/yellow]\n")
 
+    if all([o.skip() for o in sorted_outputs]):
+        console.print("All outputs skipped.\n")
+        return
+
+    full_render = command == "full-render"
+
+    if skip_fast:
+        build_pkgs = []
+
+        archs = [o0.variant["target_platform"], "noarch"]
+        for arch in archs:
+            build_pkgs += [
+                os.path.basename(x.rsplit("-", 1)[0])
+                for x in glob.glob(
+                    os.path.join(o0.config.output_folder, arch, "*.tar.bz2",)
+                )
+            ]
+
+        del_idx = []
+        for i in range(len(sorted_outputs)):
+            if f"{sorted_outputs[i].name}-{sorted_outputs[i].version}" in build_pkgs:
+                del_idx.append(i)
+
+        for idx in del_idx[::-1]:
+            console.print(
+                f"[green]Fast skip of {sorted_outputs[idx].name}-{sorted_outputs[idx].version}"
+            )
+            del sorted_outputs[idx]
+
     # Do not download source if we might skip
-    if not args.skip_existing:
+    if not (skip_existing or full_render):
         console.print("\n[yellow]Downloading source[/yellow]\n")
-        download_source(MetaData(recipe_path, o0), args.interactive)
+        download_source(MetaData(recipe_path, o0), interactive)
         cached_source = o0.sections["source"]
     else:
         cached_source = {}
+
     for o in sorted_outputs:
         console.print(
             f"\n[yellow]Preparing environment for [bold]{o.name}[/bold][/yellow]\n"
@@ -391,8 +491,15 @@ def build_recipe(args, recipe_path, cbc, config, notest: bool = False):
         meta = MetaData(recipe_path, o)
         o.set_final_build_id(meta)
 
-        if args.skip_existing:
+        if o.skip() or full_render:
+            continue
+
+        final_name = meta.dist()
+
+        # TODO this doesn't work for noarch!
+        if skip_existing:
             final_name = meta.dist()
+
             if os.path.exists(
                 os.path.join(
                     o.config.output_folder,
@@ -410,7 +517,6 @@ def build_recipe(args, recipe_path, cbc, config, notest: bool = False):
             try:
                 o.transactions["build"]["transaction"].execute(
                     PrefixData(o.config.build_prefix),
-                    o.transactions["build"]["pkg_cache"],
                 )
             except Exception:
                 # This currently enables windows-multi-build...
@@ -419,16 +525,16 @@ def build_recipe(args, recipe_path, cbc, config, notest: bool = False):
         if "host" in o.transactions:
             mkdir_p(o.config.host_prefix)
             o.transactions["host"]["transaction"].execute(
-                PrefixData(o.config.host_prefix), o.transactions["host"]["pkg_cache"],
+                PrefixData(o.config.host_prefix)
             )
 
         if cached_source != o.sections["source"]:
-            download_source(meta, args.interactive)
+            download_source(meta, interactive)
             cached_source = o.sections["source"]
 
         console.print(f"\n[yellow]Starting build for [bold]{o.name}[/bold][/yellow]\n")
 
-        final_outputs = build(meta, None, allow_interactive=args.interactive)
+        final_outputs = build(meta, None, allow_interactive=interactive)
 
         stats = {}
         if final_outputs is not None:
@@ -446,16 +552,36 @@ def build_recipe(args, recipe_path, cbc, config, notest: bool = False):
     for o in sorted_outputs:
         print("\n\n")
         console.print(o)
+    return sorted_outputs
+
+
+def extract_features(feature_string):
+    if feature_string and len(feature_string):
+        assert feature_string.startswith("[") and feature_string.endswith("]")
+        features = [f.strip() for f in feature_string[1:-1].split(",")]
+    else:
+        features = []
+
+    selected_features = {}
+    for f in features:
+        if f.startswith("~"):
+            selected_features[f[1:]] = False
+        else:
+            selected_features[f] = True
+    return selected_features
 
 
 def run_build(args):
-    folder = args.recipe_dir or os.path.dirname(args.target)
-    variant = {}
-    if args.target_platform:
-        variant["target_platform"] = args.target_platform
+    if getattr(args, "json", False):
+        global console
+        console.quiet = True
 
+    selected_features = extract_features(args.features)
+
+    folder = args.recipe_dir or os.path.dirname(args.target)
+    variant = {"target_platform": args.target_platform or context.subdir}
     cbc, config = get_config(folder, variant, args.variant_config_files)
-    cbc["target_platform"] = [args.target_platform]
+    cbc["target_platform"] = [variant["target_platform"]]
 
     if not os.path.exists(config.output_folder):
         mkdir_p(config.output_folder)
@@ -469,9 +595,13 @@ def run_build(args):
 
     for recipe in all_recipes:
         build_recipe(
-            args,
+            args.command,
             recipe["recipe_file"],
             cbc,
             config,
+            selected_features=selected_features,
             notest=getattr(args, "notest", False),
+            skip_existing=getattr(args, "skip_existing", False) != "default",
+            interactive=getattr(args, "interactive", False),
+            skip_fast=getattr(args, "skip_existing", "default") == "fast",
         )
